;**********************************************************************
;				REFUSEABLE BOX.ASM
;**********************************************************************

;DATE: 12/01/2008
;VERSION: Final Version
;XTAL TYPE:  INTERNAL 4 MHZ
;WRITTEN BY: RON NEWTON
;GENERAL REGISTER STARTS @ 20h
;INTERRUPT VECTOR = 04h
;END OF PROGRAM MEMORY = 1FFH
;COMMENTS:  THIS PROGRAM CAN PRESET THE AMP TRIPING AMPERAGE WITH A POT
;IT WORKS ON AC/DC WHEN VOLTAGE EXCEEDS PRESET 
;IT ONLY USES 256 BIT OF A/D READING THE ADRESH
;THE MULTIPLICATION OF 10 PROVIDES 3 DIGIT DISPLAY
;**********************************************************
    
    list        p=16f690        ; list directive to define processor
	RADIX		DEC				;YOU CAN CHANGE THIS TO DEC OR OCT DEFAULT IS HEX
    #include    <p16f690.inc>   ; processor specific variable definitions
    
    __CONFIG  _WDT_OFF & _PWRTE_OFF & _IESO_OFF & _CP_OFF  & _INTRC_OSC_NOCLKOUT & _BOR_OFF & _FCMEN_OFF & _CPD_OFF  

; PORT DESCRIPTIONS
;RA0 VOLTAGE READING
;RA1 NC
;RA2 DIGIT 3
;RA3 NC
;RA4 POT
;RA5 RELAY
;RB4 DIGIT 1
;RB5 DIGIT 2
;RB6 AC/DC
;RB7 PROGRAM
;RC0 A
;RC1 .
;RC2 B
;RC3 C
;RC4 E
;RC5 F
;RC6 D
;RC7 G

#DEFINE 	DIGIT_ONE		PORTB,4
#DEFINE		DIGIT_TWO		PORTB,5
#DEFINE		DIGIT_THREE		PORTA,2
#DEFINE		DC				PORTB,6
#DEFINE		PROGRAM			PORTB,7
#DEFINE		RELAY			PORTA,5
#DEFINE		FLAG1			FLAG,0


;NEW REGISTERS

TRUE  		EQU     1
FALSE 	  	EQU 	0

ONES		EQU		20H				;WORKING REGISTERS
TENS		EQU		21H
HUNDREDS	EQU		22H
COUNT		EQU		23H
COUNT1		EQU		24H
;SET_POINT	EQU		25H
COUNTER2	EQU		26H
COUNTER3	EQU		27H
FLAG		EQU		28H
TEMP		EQU		29H
HOLD		EQU		2AH
RESULT		EQU		2BH
THOUSANDS	EQU		2CH
MATH_LOW	EQU		2DH
MATH_HIGH	EQU		2EH
REMAINH		EQU		2FH
REMAINL		EQU		30H
DENOMH		EQU		31H
DENOML		EQU		32H
ACCDLO		EQU		33H
ACCDHI		EQU		34H
ACCBLO		EQU		35H
ACCBHI		EQU		36H
TENK		EQU		37H
ACCAHI		EQU		38H
ACCALO		EQU		39H
SIGN		EQU		3AH
COUNT2		EQU		3BH
AVERAGE		EQU		3CH
ONESC2		EQU		3DH
TENSC2		EQU		3EH
COUNT3		EQU		3FH
COUNT4		EQU		40H
COUNT5		EQU		41H
ONESC		EQU		42H
TENSC		EQU		43H
HUNDREDSC	EQU		44H
HUNDREDSC2	EQU		45H

	ORG		0H								;START OF PROGRAM OR RESET
	GOTO	INTIALIZING						;PUT IN NAME OF LABEL WHERE PROGRAM STARTS

	ORG		04H								;INTERRUPT VECTOR IF THERE IS ONE
	GOTO	$

INTIALIZING									;LABEL
		BANKSEL		PORTA					;MAKE SURE IN RIGHT BANK
		CLRF		PORTA					;CLEAR PORT A
		BANKSEL		TRISA					;MAKE SURE IN RIGHT BANK
		MOVLW		B'010001'				;LOADING TRISA	
 	    MOVWF		TRISA
		BANKSEL		PORTB					;MAKE SURE IN RIGHT BANK
		CLRF		PORTB					;CLEAR PORT B
		BANKSEL		TRISB					;MAKE SURE IN RIGHT BANK
		MOVLW		B'11000000'				;LOADING TRISB
		MOVWF		TRISB
		BANKSEL		PORTC					;MAKE SURE IN RIGHT BANK
		CLRF		PORTC					;CLEAR PORT C
		BANKSEL		TRISC					;MAKE SURE IN RIGHT BANK
		CLRF		TRISC					;MAKE TRISC ALL OUTPUTS
		BANKSEL		ADCON0					;MAKE SURE IN RIGHT BANK
		MOVLW		B'00000001'				;LOADING ADCON0 USING ADRESH
		MOVWF		ADCON0
		BANKSEL		ADCON1					;MAKE SURE IN RIGHT BANK
		MOVLW		B'00010000'				;LOADING ADCON1
		MOVWF		ADCON1
		BANKSEL		ANSELH					;MAKE SURE IN RIGHT BANK
		MOVLW		B'00000000'				;LOADING ANSELH
		MOVWF		ANSELH
		BANKSEL		ANSEL					;MAKE SURE IN RIGHT BANK
		MOVLW		B'00001001'				;LOADING ANSEL
		MOVWF		ANSEL
		BANKSEL		OPTION_REG				;MAKE SURE IN RIGHT BANK
		CLRF		OPTION_REG				;CLEAR OPTION REGISTER
		BANKSEL		WPUB					;MAKE SURE IN RIGHT BANK
		MOVLW		B'11000000'				;LOADING WPUB
		MOVWF		WPUB
		BANKSEL		INTCON					;MAKE SURE IN RIGHT BANK
		CLRF		INTCON					;LOADING INTCON
		BANKSEL		PORTA					;MAKE SURE IN RIGHT BANK
		MOVLW		100						;PRELOADING COUNTER2
		MOVWF		COUNTER2			
		BSF			FLAG1					;SET FOR DC
		MOVLW		255						;BLANK SEGMENTS
		MOVWF		PORTC

;***************************STARTING*****************************


START
		BSF			RELAY				;TURN OFF RELAY
		BTFSS		PROGRAM				;WATCH FOR SWITCH BEING PUSHED
		GOTO		REPROGRAM			;JUMP TO REPROGRAM
		MOVLW		74					;MOVE IN A P
		MOVWF		PORTC
		BCF			DIGIT_ONE			;TURN ON DIGIT ONE
		BSF			DIGIT_TWO			;TURN OFF DIGIT TWO
		BSF			DIGIT_THREE			;TURN OFF DIGIT THREE
		CALL		DELAY
		MOVLW		111					;MOVE IN A r
		MOVWF		PORTC		
		BSF			DIGIT_ONE			;TURN OFF DIGIT ONE
		BCF			DIGIT_TWO			;TURN ON DIGIT TWO
		BSF			DIGIT_THREE			;TURN OFF DIGIT THREE
		CALL		DELAY				;DISPLAY DELAY
		MOVLW		18					;MOVE IN A g
		MOVWF		PORTC
		BSF			DIGIT_ONE			;TURN OFF DIGIT ONE 
		BSF			DIGIT_TWO			;TURN OFF DIGIT TWO
		BCF			DIGIT_THREE			;TURN ON DIGIT THREE
		CALL		DELAY				;DISPLAY DELAY
		GOTO		START				;DO  AGAIN
						
START1	BCF			RELAY
		BTFSS		DC					;CHECK FOR DC BUTTON BEING PUSHED
		CALL		CHANGE_AC			;PUSHED GOTO CHANGE AC
		BTFSS		PROGRAM				;CHECK FOR PROGRAM BUTTON BEING PUSHED
		GOTO		REPROGRAM			;PUSHED GOTO REPROGRAM



DIRECT
		CLRF		HOLD				;CEARING AVERAGE REGISTERS
		CLRF		AVERAGE
		MOVLW		35
		MOVWF		COUNT2
		BANKSEL		ADCON0
		MOVLW		B'00000001'			;LOADING ADCON0 RA0 ADRESH ONLY
		MOVWF		ADCON0
DIR1	BSF			ADCON0,GO			;START A/D
		BTFSC		ADCON0,GO			;WAIT UNTIL FINISHED
		GOTO		$-1	
		MOVF		ADRESH,0			;MOVING ADRESH INTO W
		ADDWF		HOLD,1				;ADD TO HOLD
		BTFSS		3,0
		GOTO		DIR2				;IF NOT CARRY GO TO 
		INCF		AVERAGE,1			;IF CARRY INCREASE AVERAGE
DIR2	CALL		DISPLAY				;DISPLAY
		DECFSZ		COUNT2,1			;DECREASE COUNTER
		GOTO		DIR1				;DO AGAIN
		MOVF		AVERAGE,0			;COUNTER AT ZERO
		MOVWF		MATH_HIGH			;35 AVERAGES DONE
		MOVF		HOLD,0				;TWO BYTES MOVING INTO MATH
		MOVWF		MATH_LOW
		MOVLW		35					;DIVIDING BY 35 FOR AVERAGE
		MOVWF		DENOML
		CALL		DIVIDE				
		MOVF		MATH_LOW,0			;ONLY ONE BYTE NOW MOVE INTO AVERAGE
		MOVWF		AVERAGE
		MOVLW		5					;CORRECTING FOR OFFSET
		SUBWF		AVERAGE,1
		MOVLW		250					;CORRECTION FOR FIRST READINGS
		SUBWF		AVERAGE,0
		BTFSC		3,0
		CLRF		AVERAGE
		MOVF		AVERAGE,0			;MOVE AVERAGE INTO RESULT
		MOVWF		RESULT

;*****************************AC/DC FACTORS**************

		BTFSS		FLAG1
		GOTO		DIR3
		MOVLW		50					;SETTING DIVISION FACTOR FOR DC
		MOVWF		DENOML
		MOVLW		183					;CORRECTION MUTIPLICATION
		MOVWF		ACCBLO				;MUTIPLICATION REGISTER
		GOTO		CONT

DIR3	MOVLW		50					;SETTING DIVISION FACTOR FOR AC
		MOVWF		DENOML
		MOVLW		204					;CORRECTION MUTIPLICATION
		MOVWF		ACCBLO				;MUTIPLICATION REGISTER




CONT	MOVF		RESULT,0			;MOVING RESULT INTO ACCALO
		MOVWF		ACCALO				;DIVISION REGISTER
		CALL		MULTIPLY			;MULTIPLY BY FACTORS TO INCREASE DIVISION ACCURACY
		CALL		DIVIDE				;DIVIDE RESULT BY FACTOR FOR AC OR DC
		CALL 		DECIMAL				;CHANGE BINARY TO DECIMAL
CONT1	CALL		DISPLAY				;DISPLAY RESULT
		MOVF		ONESC,0				;MOVING CONTROLS IN TEMP REGISTERS
		MOVWF		ONESC2
		MOVF		TENSC,0
		MOVWF		TENSC2
		MOVF		HUNDREDSC,0
		MOVWF		HUNDREDSC2
		MOVF		ONES,0				;MOVE RECENT ONES INTO W
		SUBWF		ONESC2,0			;SUBTRACT CONTROL
		BTFSC		3,0					;CHECK FOR BORROW
		GOTO		CONT2
		DECF		TENSC2,1			;BORROWED DECREASE CONTROL HIGH
		MOVLW		255					;CHECK IF ROLL OVER SHOULD BE 9
		SUBWF		TENSC2,0			
		BTFSS		3,2
		GOTO		CONT2
		MOVLW		9
		MOVWF		TENSC2
CONT2	MOVF		TENS,0				;NO BORROW MOVE RECENT TENS INTO W
		SUBWF		TENSC2,0			;SUBTRACT CONTROL
		BTFSC		3,0					;CHECK FOR BORROW
		GOTO		CONT3
		DECF		HUNDREDSC2,1
		MOVLW		255					;CHECK IF ROLL OVER SHOULD BE 9
		SUBWF		HUNDREDSC2,0			
		BTFSS		3,2
		GOTO		CONT3
		GOTO		S4
CONT3	MOVF		HUNDREDS,0			;MOVE RECENT HUNDREDS INTO W
		SUBWF		HUNDREDSC2,0		;SUBTRACT CONTROL
		BTFSS		3,0					;CHECK FOR BORROW
		GOTO		S4					;NUMBER LARGER THAN CONTROL
		GOTO		START1				;DID NOT EXCEED GO TO START

;*******************************BAD ************************

S4		BSF			RELAY				;EXCEEDED TURNED OFF RELAY
		BSF			DIGIT_ONE			;BLANKING LED
		BSF			DIGIT_TWO
		BSF			DIGIT_THREE
		MOVLW		7					;MOVE b IN TO DIGIT ONE
		MOVWF		PORTC				
		BCF			DIGIT_ONE			;TURN ON DIGIT ONE
		CALL		DELAY				;DISPLAY FOR 8 MILLISECONDS
		MOVLW		66					;DISPLAY A
		MOVWF		PORTC
		BSF			DIGIT_ONE
		BCF			DIGIT_TWO
		CALL		DELAY				;DISPLAY FOR 8 MILLISECONDS
		MOVLW		35					;DISPLAY D
		MOVWF		PORTC
		BSF			DIGIT_TWO
		BCF			DIGIT_THREE
		CALL		DELAY				;DISPLAY FOR 8 MILLISECONDS
		BTFSS		PROGRAM				;WATCH FOR SWITCH BEING PUSHED
		GOTO		REPROGRAM			;JUMP TO REPROGRAM

		GOTO		S4					;ENDLESS LOOP MUST TURN OFF POWER TO START


;*******************************************************************
;                       Test Program
;*******************************************************************
;    Load constant values to ACCa & ACCb for testing
;

MULTIPLY

	movlw   0
	movwf   ACCAHI
	movlw   0
	movwf   ACCBHI
	call    D_mpyF          ; Here (ACCb,ACCc) = 00FF 7E01
	RETURN

mulMac  MACRO
	LOCAL   NO_ADD
;
	rrf     ACCDHI, F       ;rotate d right
	rrf     ACCDLO, F
	btfss   STATUS,C    	;need to add?
	goto    NO_ADD          ; no addition necessary
	movf    ACCALO,W        ; Addition ( ACCb + ACCa -> ACCb )
	addwf   ACCBLO, F       ;add lsb
	btfsc   STATUS,C    	;add in carry
	incf    ACCBHI, F
	movf    ACCAHI,W
	addwf   ACCBHI, F       ;add msb
NO_ADD  rrf     ACCBHI, F
	rrf     ACCBLO, F
	rrf     MATH_HIGH, F
	rrf     MATH_LOW, F
;
	ENDM
;
;*******************************************************************;
;               Double Precision Multiply ( 16x16 -> 32 )
;         ( ACCb*ACCa -> ACCb,ACCc ) : 32 bit output with high word
;  in ACCb ( ACCBHI,ACCBLO ) and low word in ACCc ( MATH_HIGH,MATH_LOW ).
;
D_mpyF                          ;results in ACCb(16 msb's) and ACCc(16 lsb's)
	call    setup
;
; use the mulMac macro 16 times
;
	mulMac
	mulMac
	mulMac
	mulMac
	mulMac
	mulMac
	mulMac
	mulMac
	mulMac
	mulMac
	mulMac
	mulMac
	mulMac
	mulMac
	mulMac
	mulMac
	RETURN
 
;*******************************************************************
;
setup   movlw   .16             ; for 16 shifts
	movwf   TEMP
	movf    ACCBHI,W          	;move ACCb to ACCd
	movwf   ACCDHI
	movf    ACCBLO,W
	movwf   ACCDLO
	clrf    ACCBHI
	clrf    ACCBLO
	retlw   0

;*********************BINARY TO BCD******************************
		
DECIMAL

		SWAPF		MATH_HIGH,W
		ANDLW		0FH
		ADDLW		0F0H
		MOVWF		THOUSANDS
		ADDWF		THOUSANDS,F
		ADDLW		0E2H
		MOVWF		HUNDREDS
		ADDLW		32H
		MOVWF		ONES

		MOVF		MATH_HIGH,0
		ANDLW		0FH
		ADDWF		HUNDREDS,F
		ADDWF		HUNDREDS,F
		ADDWF		ONES,F
		ADDLW		0E9H
		MOVWF		TENS
		ADDWF		TENS,F
		ADDWF		TENS,F

		SWAPF		MATH_LOW,W
		ANDLW		0FH
		ADDWF		TENS,F
		ADDWF		ONES,F

		RLF			TENS,F
		RLF			ONES,F
		COMF		ONES,F
		RLF			ONES,F
			
		MOVF		MATH_LOW,W
		ANDLW		0FH
		ADDWF		ONES,F
		RLF			THOUSANDS,F
		
		MOVLW		07H
		MOVWF		TENK

		MOVLW		0AH
LB1		ADDWF		ONES,F
		DECF		TENS,F
		BTFSS		3,0
		GOTO		LB1
LB2		ADDWF		TENS,F
		DECF		HUNDREDS,F
		BTFSS		3,0
		GOTO		LB2

LB3		ADDWF		HUNDREDS,F
		DECF		THOUSANDS,F
		BTFSS		3,0
		GOTO		LB3

LB4		ADDWF		THOUSANDS,F
		DECF		TENK,F
		BTFSS		3,0
		GOTO		LB4

		RETURN

;******************************DIVIDE CODE*********************

;Double Precision Division
;FROM APPLICATION NOTE AN525
;      (a) Load the Denominator in location DENOMH & DENOML ( 16 bits )
;      (b) Load the Numerator in location MATH_HIGH & MATH_LOW ( 16 bits )
;      (c) CALL D_div
;      (d) The 16 bit result is in location MATH_HIGH & MATH_LOW
;      (e) The 16 bit Remainder is in locations REMAINH & REMAINL
;
;*******************************************************************;
																																																																																				;

; START WITH THIS CODE FOR DIVISION
DIVIDE    
	movlw   0				;LOADING NUMERATOR AND DENOMINATOR
	movwf   DENOMH
	call    D_divF         
 	MOVF	MATH_HIGH,0
	MOVWF	MATH_HIGH
	MOVF	MATH_LOW,0
	MOVWF	MATH_LOW
	RETURN				;RESULTS IN MATH HIGH AND MATH LOW  REMAINDER IN REMAIN HIGH ANDLOW REGISTERS

divMac  MACRO
	LOCAL   NOCHK
	LOCAL   NOGO
	bcf     STATUS,C
	rlf     ACCDLO, F
	rlf     ACCDHI, F
	rlf     REMAINL, F
	rlf     REMAINH, F
	movf    DENOMH,W
	subwf   REMAINH,W          ;check if a>c
	btfss   STATUS,Z
	goto    NOCHK
	movf    DENOML,W
	subwf   REMAINL,W        ;if msb EQUal then check lsb
NOCHK   
	btfss   STATUS,C        ;carry set if c>a
	goto    NOGO
	movf    DENOML,W        ;c-a into c
	subwf   REMAINL, F
	btfss   STATUS,C
	decf    REMAINH, F
	movf    DENOMH,W
	subwf   REMAINH, F
	bsf     STATUS,C        ;shift a 1 into b (result)
NOGO
    rlf     MATH_LOW, F
	rlf     MATH_HIGH, F
	ENDM

setup1   
	movlw   .16             ; for 16 shifts
	movwf   TEMP
	movf    MATH_HIGH,W        ;move ACCb to ACCd
	movwf   ACCDHI
	movf    MATH_LOW,W
	movwf   ACCDLO
	clrf    MATH_HIGH
	clrf    MATH_LOW
	retlw   0

D_divF
	call    setup1
	clrf    REMAINH
	clrf    REMAINL

; use the mulMac macro 16 times

	divMac
	divMac
	divMac
	divMac
	divMac
	divMac
	divMac
	divMac
	divMac
	divMac
	divMac
	divMac
	divMac
	divMac
	divMac
	divMac
	RETURN

;*********************END DIVISION***********************


CHANGE_AC
		BTFSS		DC					;WAITING FOR DC SWITCH TO BE RELEASED
		GOTO		$-1
		BSF			DIGIT_ONE			;TURNING OFF DIGITS
		BSF			DIGIT_TWO
		BSF			DIGIT_THREE
		BTFSS		FLAG1				;CHECK FOR FLAG ONE BEING SET
		GOTO		DISPLAY_DC			;NOT SET GO TO DISPLAY DC

AC		BCF			FLAG1				;SET CLEAR FLAG
AC1		MOVLW		66					;DISPLAY A
		MOVWF		PORTC
		BSF			DIGIT_ONE
		BCF			DIGIT_TWO
		CALL 		DELAY				;DISPLAY FOR 8 MILLISECONDS
		MOVLW		142					;DISPLAY C
		MOVWF		PORTC
		BSF			DIGIT_TWO
		BCF			DIGIT_THREE
		CALL		DELAY				;DISPLAY FOR 8 MILLISECONDS
		BSF			DIGIT_THREE
		DECFSZ		COUNTER2,1			;DISPLAY FOR 3 SECONDS AC
		GOTO		AC1
		MOVLW		100
		MOVWF		COUNTER2
		RETURN

DISPLAY_DC								;LABEL
DC1		BSF			FLAG1				;SET THE FLAG1
		MOVLW		35					;DISPLAY D
		MOVWF		PORTC
		BSF			DIGIT_ONE
		BSF			DIGIT_THREE
		BCF			DIGIT_TWO
		CALL 		DELAY				;DISPLAY FOR 8 MILLISECONDS
		MOVLW		142
		MOVWF		PORTC				;DISPLAY C
		BSF			DIGIT_TWO
		BCF			DIGIT_THREE
		CALL		DELAY				;DISPLAY FOR 8 MILLISECONDS
		BSF			DIGIT_THREE
		DECFSZ		COUNTER2,1			;DISPLAY DC FOR 3 SECONDS
		GOTO	 	DC1
		MOVLW		100
		MOVWF		COUNTER2
		RETURN

DELAY   MOVLW	10						;EIGHT MILLISECOND DELAY FOR DISPLAYING DIGITS
		MOVWF	COUNT1
D1		DECFSZ	COUNT,1
		GOTO	D1

		DECFSZ	COUNT1,1
		GOTO	D1
		RETURN 

REPROGRAM


		BSF			RELAY				;TURN OFF RELAY
		BANKSEL		ADCON0				;MAKE SURE WE ARE IN THE RIGHT BANK
		MOVLW		B'00001101'			;CHANGING A/D PORT AN3 RA4
		MOVWF		ADCON0
		BSF			ADCON0,GO			;START A/D
		BTFSC		ADCON0,GO			;WAIT UNTIL FINISHED
		GOTO		$-1
		MOVF		ADRESH,0			;MEASURING POT
		MOVWF		ACCALO
		MOVLW		10					;AMPLIFY BY 10
		MOVWF		ACCBLO
		CALL		MULTIPLY
		MOVLW		5					;FACTOR FOR POT
		MOVWF		DENOML
		CALL		DIVIDE				;DIVIDE
		CALL		DECIMAL				;CONVERT FROM HEX TO DECIMAL
		MOVF		ONES,0				;MOVING ONES INTO CONTROL
		MOVWF		ONESC
		MOVF		TENS,0				;MOVING TENS INTO CONTROL
		MOVWF		TENSC
		MOVF		HUNDREDS,0			;MOVING HUNDREDS INTO CONTROL
		MOVWF		HUNDREDSC
		CALL		DISPLAY				;DISPLAY ALL THREE DIGITS
		BTFSS		PROGRAM				;CHECK TO SEE IF PROGRAM IS RELEASED
		GOTO		REPROGRAM			;NOT RELEASE DO AGAIN
		BTFSS		FLAG1				;CHECK FOR AC TO DC BY FLAG
		GOTO		RE1					;FLAG CLEAR GOTO TO RE1
		CALL		DC1					;FLAG SET GO TO DC1
		GOTO		START1				;RETURN TO START

RE1		CALL		AC1					;NOT SET GO TO AC1
		GOTO		START1				;RETURN TO START

DISPLAY
		MOVLW		255				;BLANK PORTC
		MOVWF		PORTC			
		MOVF		ONES,0			;MOVE ONES INTO W
		BCF			DIGIT_THREE		;TURN OFF DIGIT THREE
		BSF			DIGIT_TWO		;TURN OFF DIGIT TWO
		BSF			DIGIT_ONE		;TURN OFF DIGIT ONE
		MOVLW		0				;LOAD INTO ADRES THE LOW ADDRESS OF LOOKUP TABLE LOOKUP TABLE ADDRESS 
		ADDWF		ONES,0			;ADD THE OFFSET TO THE LOE ADDRESS (ADRES)
		MOVLW		5				;LOAD 5 INTO HIGH ADDRESS OF LOOKUP TABLE
		BTFSC		STATUS,C		;CHECK TO SEE IF THE OFFSET ADDED TO LOW LOOKUP TABLE WENT ABOVE 255
		ADDLW		1				;IF IT DID ADD 1 TO HIGH TABLE BYTE (PCLATH)
		MOVWF		PCLATH			;MOVE ONE INTO PCLATH REGISTER
		MOVF		ONES,0			;LOAD OFFSET INTO W
		CALL		PATTERN			;JUMP TO PATTERN
		CLRF		PCLATH
		MOVWF		PORTC			;MOVE PATTERN INTO PORTC
		BCF			DIGIT_THREE		;TURN ON DIGIT THREE
		CALL		DELAY			;CALL THIS LABEL. THIS DELAY ALLOWS FOR BETTER DISPLAYING
	
		MOVF		TENS,0			;MOVE TENS INTO W
		BSF			DIGIT_ONE		;TURN OFF DIGIT ONE
		BSF			DIGIT_TWO		;TURN OFF DIGIT TWO
		BSF			DIGIT_THREE
		MOVLW		0				;LOAD INTO ADRES THE LOW ADDRESS OF LOOKUP TABLE LOOKUP TABLE ADDRESS 
		ADDWF		TENS,0			;ADD THE OFFSET TO THE LOW ADDRESS (ADRES)
		MOVLW		5				;LOAD 5 INTO HIGH ADDRESS OF LOOKUP TABLE
		BTFSC		STATUS,C		;CHECK TO SEE IF THE OFFSET ADDED TO LOW LOOKUP TABLE WENT ABOVE 255
		ADDLW		1				;IF IT DID ADD 1 TO HIGH TABLE BYTE (PCLATH)
		MOVWF		PCLATH			;MOVE ONE INTO PCLATH REGISTER
		MOVF		TENS,0			;LOAD OFFSET INTO W
		CALL 		PATTERN			;CALL PATTERN AND RETURN
		CLRF		PCLATH
		MOVWF		PORTC			;MOVE WHAT WAS IN PATTERN TO PORT B
		BCF			DIGIT_TWO		;TURN ON DIGIT TWO
		CALL		DELAY			;CALL THIS LABEL. THIS DELAY ALLOWS FOR BETTER DISPLAYING		
	
		MOVF		HUNDREDS,0		;MOVE HUNDREDS INTO W
		BSF			DIGIT_ONE		;TURN OFF DIGIT ONE
		BSF			DIGIT_TWO		;TURN OFF DIGIT TWO
		BSF			DIGIT_THREE
		MOVWF		0				;LOAD INTO ADRES THE LOW ADDRESS OF LOOKUP TABLE LOOKUP TABLE ADDRESS 
		ADDWF		HUNDREDS,0		;ADD THE OFFSET TO THE LOE ADDRESS (ADRES)
		MOVLW		5				;LOAD 5 INTO HIGH ADDRESS OF LOOKUP TABLE
		BTFSC		STATUS,C		;CHECK TO SEE IF THE OFFSET ADDED TO LOW LOOKUP TABLE WENT ABOVE 255
		ADDLW		1				;IF IT DID ADD 1 TO HIGH TABLE BYTE (PCLATH)
		MOVWF		PCLATH			;MOVE ONE INTO PCLATH REGISTER
		MOVF		HUNDREDS,0		;LOAD OFFSET INTO W
		CALL 		PATTERN			;CALL PATTERN AND RETURN
		MOVWF		PORTC			;MOVE WHAT WAS IN PATTERN TO PORT B
		CLRF		PCLATH			;SET PCLATH TO ZERO
		BCF			PORTC,1			;TURN ON DECIMAL
		BCF			DIGIT_ONE		;TURN ON DIGIT ONE
		CALL		DELAY			;CALL THIS LABEL. THIS DELAY ALLOWS FOR BETTER DISPLAYING		
		RETURN						;REPEAT			


;*******************PATTERN LOOK UP TABLE**************************
	ORG	500h
PATTERN							;LABEL 


		ADDWF	PCL,1			;ADD W TO PROGRAM COUNTER AND LOOK UP DATA
		RETLW	130				;0
		RETLW	243				;1
		RETLW	42				;2
		RETLW	50				;3
		RETLW	83				;4
		RETLW	22				;5
		RETLW	6				;6
		RETLW	242				;7
		RETLW	2				;8
		RETLW	82				;9
    END                       ; directive 'end of program'